# -*- coding: utf-8 -*-

__all__ = ['is_image_file',
           ]


def is_image_file(filename):
    """ Check if given file is image file or not

    Parameters
    -------
    filename: str
        input file path

    Returns
    -------
    img_flag: bool
        flag for image

    """

    IMG_EXTENSIONS = [
        '.jpg', '.JPG', '.jpeg', '.JPEG',
        '.png', '.PNG',
        '.ppm', '.PPM',
        '.bmp', '.BMP',
        'tif', 'TIF', 'tiff', 'TIFF',
    ]

    img_flag = any(filename.endswith(extension) for extension in IMG_EXTENSIONS)

    return img_flag



# -*- coding: utf-8 -*-

import deepdish as dd

__all__ = ['h5_to_dict',
           'dict_to_h5'
           ]

def h5_to_dict(h5_path):
    """ Load h5 to python dictionary.

    Parameters
    -------
    h5_path: str
        h5 file path

    Returns
    -------
    data_dict: dict
        data represented as python dictionary

    """

    data_dict = dd.io.load(h5_path)

    return data_dict


def dict_to_h5(data_dict, h5_path):
    """ Save python dictionary to h5 file.

    Parameters
    -------
    data_dict: dict
        data represented as dictionary
    h5_path: str
        h5 file path to save

    """

    dd.io.save(h5_path, data_dict)



# -*- coding: utf-8 -*-

import os
import pickle

__all__ = ['pkl_to_dict',
           'dict_to_pkl'
           ]

def pkl_to_dict(pkl_path):
    """ Load pickle file as python dictionary

    Parameters
    -------
    pkl_path: str
        pickle file path

    Returns
    -------
    data_dict: dict
        data represented as python dictionary

    """

    if not os.path.exists(pkl_path):
        raise AssertionError("{} not exist".format(pkl_path))

    with open(pkl_path, 'rb') as handle:
        data_dict = pickle.load(handle)

    return data_dict


def dict_to_pkl(data_dict, pkl_path):
    """ Save dictionary data as pickle file

    Parameters
    -------
    data_dict: dict
        data represented as dictionary
    pkl_path: str
        pickle file path to save

    """

    with open(pkl_path, 'wb') as handle:
        pickle.dump(data_dict, handle, protocol=pickle.HIGHEST_PROTOCOL)



# -*- coding: utf-8 -*-

from datetime import datetime

__all__ = ["time_to_str",
           "current_time"]


def current_time():
    """ Provide current time as YYYY-MM-DD_HH-MM-SS

    Parameters
    --------
    None

    Returns
    --------
    time_str: str
        current time string

    """

    time_str = datetime.now().strftime('%Y-%m-%d_%H-%M-%S')

    return time_str


def time_to_str(delta_t, mode="min"):
    """Convert elapsed time to string representation

    Parameters
    --------
    delta_t: time difference
        Elapsed time
    mode: str
        Time representation manner, by "minitues" or "seconds".

    Returns
    --------
    delta_str: str
        Elapsed time string representation

    Examples
    --------
    >>> from timeit import default_timer as timer
    >>> start = timer()
    >>> end = timer()
    >>> elapsed_time_str = time_to_str(end - start, "min")

    """
    if mode == "min":
        t = int(delta_t) / 60
        hrs = int(t // 60)
        mins = int(t % 60)
        delta_str = "{:2d} hr {:2d} min".format(hrs, mins)
    elif mode == "sec":
        t = int(delta_t)
        mins = int(t // 60)
        sec = int(t % 60)
        delta_str = "{:2d} min {:2d} sec".format(mins, sec)
    else:
        raise NotImplementedError()

    return delta_str



# -*- coding: utf-8 -*-

import os, sys

import subprocess
import tempfile
import itertools


# path to the stanford corenlp jar
STANFORD_CORENLP_3_4_1_JAR = 'stanford-corenlp-3.4.1.jar'

# punctuations to be removed from the sentences
PUNCTUATIONS = ["''", "'", "``", "`", "-LRB-", "-RRB-", "-LCB-", "-RCB-", \
        ".", "?", "!", ",", ":", "-", "--", "...", ";"]


class PTBTokenizer:
    """Python wrapper of Stanford PTBTokenizer"""

    def tokenize(self, report):
        cmd = ['java', '-cp', STANFORD_CORENLP_3_4_1_JAR, \
                'edu.stanford.nlp.process.PTBTokenizer', \
                '-preserveLines', '-lowerCase']

        # ======================================================
        # save sentences to temporary file
        # ======================================================
        path_to_jar_dirname=os.path.dirname(os.path.abspath(__file__))
        tmp_file = tempfile.NamedTemporaryFile(delete=False, dir=path_to_jar_dirname)
        tmp_file.write(report.strip().encode())
        tmp_file.close()

        # ======================================================
        # tokenize sentence
        # ======================================================
        cmd.append(os.path.basename(tmp_file.name))
        p_tokenizer = subprocess.Popen(cmd, cwd=path_to_jar_dirname, \
                stdout=subprocess.PIPE)
        tokenized_report = p_tokenizer.communicate(input=report)[0]
        # remove temp file
        os.remove(tmp_file.name)

        tokenized_report = tokenized_report.decode()
        report_tokens = [w for w in tokenized_report.split(' ') \
                         if w not in PUNCTUATIONS]

        return report_tokens



# -*- coding: utf-8 -*-

import numpy as np

__all__ = ['PIL2npArr',
           'Bool2Uint8',
           'Binary2Bool'
           ]


def PIL2npArr(pil_img):
    """ Convert Pillow image to numpy array.

    """

    np_img = np.asarray(pil_img)
    if len(np_img.shape) == 2:
        pass
    elif len(np_img.shape) == 3:
        if np_img.shape[2] == 4:
            np_img = np_img[...,:-1]
    else:
        raise NotImplementedError()

    return np_img


def Bool2Uint8(bool_img):
    """ Convert bool image to dtype uint8.

    """

    u_img = bool_img.astype(np.uint8)

    return u_img


def Binary2Bool(bin_img):
    """ Convert binary image to dtype bool.

    """

    max_val = np.amax(bin_img)
    bool_img = (bin_img / max_val).astype('bool')

    return bool_img



# -*- coding: utf-8 -*-

__all__ = ['txt_to_list',
           'list_to_txt'
           ]

def txt_to_list(txt_path):
    """ Load text file as list

    Parameters
    -------
    txt_path: str
        text file path

    Returns
    -------
    data_list: list
        data represented as python list

    """

    with open(txt_path, 'r') as file:
      data_list = [line.rstrip('\n') for line in file]

    return data_list


def list_to_txt(data_list, txt_path):
    """ Save python list to text file

    Parameters
    -------
    data_list: list
        data represented as python list
    txt_path: str
        text file path to save

    """

    with open(txt_path, 'w') as file:
        for item in data_list:
            file.write("{}\n".format(item))



# -*- coding: utf-8 -*-

import json

__all__ = ['json_to_dict',
           'dict_to_json'
           ]


def json_to_dict(json_path):
    """ Load json file as dictionary

    Parameters
    -------
    json_path: str
        json file path

    Returns
    -------
    data_dict: dict
        data represented as python dictionary

    """

    with open(json_path) as fp:
        data_dict = json.load(fp)

    return data_dict


def dict_to_json(data_dict, json_path):
    """ Save dictionary to json file

    Parameters
    -------
    data_dict: dict
        data represented as dictionary
    json_path: str
        json file path to save

    """

    with open(json_path, 'w') as fp:
        json.dump(data_dict, fp)



# -*- coding: utf-8 -*-

import sys

__all__ = ['Logger',
           ]

class Logger(object):
    """ Logging to terminal and file.

    """
    def __init__(self):
        self.terminal = sys.stdout
        self.file = None

    def open(self, file, mode=None):
        if mode is None:
            mode ='w'
        self.file = open(file, mode)

    def write(self, message, is_terminal=1, is_file=1):
        if '\r' in message:
            is_file = 0

        if is_terminal == 1:
            self.terminal.write(message)
            self.terminal.flush()

        if is_file == 1:
            self.file.write(message)
            self.file.flush()

    def flush(self):
        pass



# -*- coding: utf-8 -*-

import os

__all__ = ['find_ext_files',
           ]


def find_ext_files(dir_name, ext):
    """ Find all files with given extension in a given directory

    Parameters
    -------
    dir_name: str
        given directory to locate files
    ext: str
        file extension

    Returns
    -------
    file_list: list
        files with specified extesion in given directory

    """
    if not os.path.isdir(dir_name):
        raise AssertionError("{} is not a valid directory".format(dir_name))

    file_list = []
    for root, _, files in os.walk(dir_name):
        for cur_file in files:
            if cur_file.endswith(ext):
                 file_list.append(os.path.join(root, cur_file))

    return file_list



# -*- coding: utf-8 -*-

import os
import glob, shutil, uuid

__all__ = ['batch_uuid_rename', 'batch_rename_files',
           ]


def batch_rename_files(input_dir, save_dir, ext='.png', start_num=0, filename_len=5):
    """ Rename files with ordered filenames

    """

    all_files = glob.glob(os.path.join(input_dir, "*" + ext))

    # Delete out directory and create a new one
    if os.path.exists(save_dir):
        shutil.rmtree(save_dir)
    os.makedirs(save_dir)

    for ind, filepath in enumerate(all_files):
        cur_ind = start_num + ind
        save_filename = str(cur_ind).zfill(filename_len) + ext
        save_path = os.path.join(save_dir, save_filename)
        shutil.copy(filepath, save_path)


def batch_uuid_rename(input_dir, save_dir, ext=".png"):
    """ Rename files with uuid names to deidentify

    """

    all_files = glob.glob(os.path.join(input_dir, "*" + ext))
    # Delete out directory and create a new one
    if os.path.exists(save_dir):
        shutil.rmtree(save_dir)
    os.makedirs(save_dir)

    for filepath in all_files:
        cur_uuid = uuid.uuid4()
        save_filename = str(cur_uuid)[:8] + ext
        save_path = os.path.join(save_dir, save_filename)
        shutil.copy(filepath, save_path)



# -*- coding: utf-8 -*-

import os
import shutil

__all__ = ['overwrite_dir',
           'check_mkdir',
           ]

def overwrite_dir(dir_name):
    """ Overwrite directory if exist, create new directory

    """

    if os.path.exists(dir_name):
        shutil.rmtree(dir_name)
    os.makedirs(dir_name)



def check_mkdir(dir_name):
    """ if directory not exist, create the directory

    """

    if not os.path.exists(dir_name):
        os.makedirs(dir_name)



# -*- coding: utf-8 -*-

import pandas as pd

__all__ = ['csv_to_dict',
           'dict_to_csv'
           ]

def csv_to_dict(csv_path):
    """ Load csv to python dictionary.

    Parameters
    -------
    csv_path: str
        csv file path

    Returns
    -------
    data_dict: dict
        data represented as python dictionary

    """

    df = pd.read_csv(csv_path)
    data_dict = df.to_dict('list')

    return data_dict


def dict_to_csv(data_dict, csv_path):
    """ Save python dictionary to csv file.

    Parameters
    -------
    data_dict: dict
        data represented as dictionary
    csv_path: str
        csv file path to save

    """

    df = pd.DataFrame.from_dict(data_dict)
    df.to_csv(csv_path, index=False)
